
/*
 * Script to change a keyword value in a set of FITS files.
 *
 * Original routine code: Juan Conejero (PTeam)
 * http://pixinsight.com/forum/index.php?topic=9721.msg61787#msg61787
 *
 * GUI: Enzo De Bernardini (PI User)
 *
 * ** All credits to Juan Conejero (PTeam) **
 *
 * Changelog:
 * 1.0.0: First release in PixInsight Forum - Juan Conejero
 *        GUI Added - Enzo De Bernardini
 * 1.0.1: Remove requiered oldKeywordValue
 * 1.0.2: Remove all reference to old keyword value;
 *        change file type to .xisf;
 *        - Fred Vanner
 *
*/

var inputDirectory = "";
var outputDirectory = "";
var keywordName = "";
var newKeywordValue = "";

#feature-id    Batch Processing > BatchEditSingleFITSHeader

#define TITLE  "BatchEditSingleFITSHeader"
#define VERSION "1.0.2"

#include <pjsr/Sizer.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/StdButton.jsh>
#include <pjsr/StdIcon.jsh>
#include <pjsr/UndoFlag.jsh>


function EditKey()
{
   this.executeMainFunction = function()
   {
      if( this.inputDirectory == undefined || this.inputDirectory == "" || this.outputDirectory == "" || this.outputDirectory == undefined || this.keywordName == "" || this.keywordName == undefined || this.newKeywordValue == "" || this.newKeywordValue == undefined )
      {
         var msg = new MessageBox( "All fields must be complete.", "Script Settings Warning", StdIcon_Warning, StdButton_Ok );
         msg.execute();
         return false;
      }
      else
      {
         BatchEditKey( this.inputDirectory, this.outputDirectory, this.keywordName, this.newKeywordValue );
         return true;
      }
   }; // executeMainFunction()
}; // Write


function BatchEditKey( inputDirectory, outputDirectory, keywordName, newKeywordValue )
{
   if ( inputDirectory.length == 0 )
      throw new Error( "Must specify an input directory" );
   if ( outputDirectory.length == 0 )
      throw new Error( "Must specify an output directory" );
   if ( keywordName.length == 0 )
      throw new Error( "Must specify a keyword name" );
   if ( newKeywordValue.length == 0 )
      throw new Error( "Must specify a new keyword values" );

   /*
   if ( oldKeywordValue.length == 0 || newKeywordValue.length == 0 )
      throw new Error( "Must specify old and new keyword values" );
   */

   if ( !inputDirectory.endsWith( '/' ) )
      inputDirectory += '/';
   if ( !outputDirectory.endsWith( '/' ) )
      outputDirectory += '/';
   for ( let D = searchDirectory( inputDirectory + "*.xisf", false/*recursive*/ ), i = 0; i < D.length; ++i )
   {
      let w = ImageWindow.open( D[i] );
      let k = w[0].keywords;
      let found = false;
      for ( let j = 0; j < k.length; ++j )
      {
         k[j].trim();
         if ( k[j].name == keywordName )
            {
               k[j].value = newKeywordValue;
               found = true;
               break;
            }
      }
      if ( found )
      {
         console.writeln( "<end><cbr>" + D[i] );
         w[0].keywords = k;
         w[0].saveAs( outputDirectory + File.extractNameAndSuffix( D[i] ),
                      false/*queryOptions*/,
                      false/*allowMessages*/,
                      false/*strict*/,
                      true/*verifyOverwrite*/ );
      }
      w[0].forceClose();
   }
}

// GUI

var engine = new EditKey;

function wj_dialog() {

   this.__base__ = Dialog;
   this.__base__();

   this.helpLabel = new Label (this);
   this.helpLabel.frameStyle = FrameStyle_Box;
   this.helpLabel.margin = 4;
   this.helpLabel.wordWrapping = true;
   this.helpLabel.useRichText = true;
   this.helpLabel.text = "<b>" + TITLE + " v" + VERSION + "</b> &mdash; Batch Edit Single FITS header";
   this.helpLabel.setMinWidth( 500 );


   // Keyword Name

   this.keywordName_Label = new Label( this );
   this.keywordName_Label.text = "Target Keyword";
   this.keywordName_Label.textAlignment = TextAlign_Left|TextAlign_VertCenter;
   this.keywordName_Label.toolTip = "Target Keyword Name to batch edit.";

   this.keywordName_Edit = new Edit( this );
   this.keywordName_Edit.toolTip = "Target Keyword Name to batch edit.";

   this.keywordName_Edit.onEditCompleted = function()
   {
      engine.keywordName = this.text;
   };


   // New Keyword Value

   this.newKeywordValue_Label = new Label( this );
   this.newKeywordValue_Label.text = "New Keyword Value";
   this.newKeywordValue_Label.textAlignment = TextAlign_Left|TextAlign_VertCenter;
   this.newKeywordValue_Label.toolTip = "New Keyword Value";

   this.newKeywordValue_Edit = new Edit( this );
   this.newKeywordValue_Edit.toolTip = "New Keyword Value.";

   this.newKeywordValue_Edit.onEditCompleted = function()
   {
      engine.newKeywordValue = this.text;
   };




   this.fileSettings_GroupBox = new GroupBox( this );
   this.fileSettings_GroupBox.title = "Settings";
   this.fileSettings_GroupBox.sizer = new VerticalSizer;
   this.fileSettings_GroupBox.sizer.margin = 6;
   this.fileSettings_GroupBox.sizer.spacing = 4;
   this.fileSettings_GroupBox.sizer.addStretch();

   this.fileSettings_GroupBox.sizer.add( this.keywordName_Label, 10 );
   this.fileSettings_GroupBox.sizer.add( this.keywordName_Edit );
   this.fileSettings_GroupBox.sizer.addStretch();

   this.fileSettings_GroupBox.sizer.add( this.newKeywordValue_Label, 50 );
   this.fileSettings_GroupBox.sizer.add( this.newKeywordValue_Edit );
   this.fileSettings_GroupBox.sizer.addStretch();


   // Input Directory

   this.InputDir_Edit = new Edit( this );
   this.InputDir_Edit.readOnly = true;
   this.InputDir_Edit.text = "";
   this.InputDir_Edit.toolTip = "<p>Input directory.</p>";

   this.InputDir_Select_Button = new PushButton( this );
   this.InputDir_Select_Button.text = " Input Directory ";
   this.InputDir_Select_Button.toolTip = "<p>Select the input directory.</p>";

#ifneq __PI_PLATFORM__ MACOSX
   this.InputDir_Select_Button.icon = new Bitmap( ":/browser/select-file-1.png" );
#endif

   this.InputDir_Select_Button.onClick = function()
   {
      var gdd = new GetDirectoryDialog;
      gdd.caption = "Select Input Directory";
      if ( gdd.execute() )
      {
         engine.inputDirectory = gdd.directory;
         this.dialog.InputDir_Edit.text = engine.inputDirectory;
      }
   };

   // If available, get the current file path
   var currentPath = File.extractDrive( ImageWindow.activeWindow.filePath ) + File.extractDirectory( ImageWindow.activeWindow.filePath );
   this.dialog.InputDir_Edit.text = currentPath;
   engine.inputDirectory = currentPath;


   // Output Directory

   this.OutputDir_Edit = new Edit( this );
   this.OutputDir_Edit.readOnly = true;
   this.OutputDir_Edit.text = "";
   this.OutputDir_Edit.toolTip = "<p>Output directory.</p>";

   this.OutputDir_Select_Button = new PushButton( this );
   this.OutputDir_Select_Button.text = " Output Directory ";
   this.OutputDir_Select_Button.toolTip = "<p>Select the output directory.</p>";

#ifneq __PI_PLATFORM__ MACOSX
   this.OutputDir_Select_Button.icon = new Bitmap( ":/browser/select-file-1.png" );
#endif

   this.OutputDir_Select_Button.onClick = function()
   {
      var gdd = new GetDirectoryDialog;
      gdd.caption = "Select Output Directory";
      if ( gdd.execute() )
      {
         engine.outputDirectory = gdd.directory;
         this.dialog.OutputDir_Edit.text = engine.outputDirectory;
      }
   };


   this.Paths_GroupBox = new GroupBox( this );
   this.Paths_GroupBox.title = "Files Path";

   this.Paths_GroupBox.sizer = new VerticalSizer;
   this.Paths_GroupBox.sizer.margin = 6;
   this.Paths_GroupBox.sizer.spacing = 4;

   this.Paths_GroupBox.sizer.add( this.InputDir_Edit, 100 );
   this.Paths_GroupBox.sizer.add( this.InputDir_Select_Button );

   this.Paths_GroupBox.sizer.spacing = 6;

   this.Paths_GroupBox.sizer.add( this.OutputDir_Edit, 100 );
   this.Paths_GroupBox.sizer.add( this.OutputDir_Select_Button );


   // Buttons

   this.ok_Button = new PushButton (this);
   this.ok_Button.text = "OK";

#ifneq __PI_PLATFORM__ MACOSX
   this.ok_Button.icon = new Bitmap( ":/images/icons/ok.png" );
#endif

   this.ok_Button.onClick = function() {
      console.show();
      var result = engine.executeMainFunction();
      if( result )
      {
         this.dialog.cancel();
      };
   };

   this.cancel_Button = new PushButton (this);
   this.cancel_Button.text = "Cancel";

#ifneq __PI_PLATFORM__ MACOSX
   this.cancel_Button.icon = new Bitmap( ":/images/icons/cancel.png" );
#endif

   this.cancel_Button.onClick = function() {
      console.hide();
      this.dialog.cancel();
   };

   this.buttons_Sizer = new HorizontalSizer;
   this.buttons_Sizer.spacing = 6;
   this.buttons_Sizer.addStretch();
   this.buttons_Sizer.add ( this.ok_Button );
   this.buttons_Sizer.add ( this.cancel_Button );
   this.buttons_Sizer.addStretch();

   this.sizer = new VerticalSizer;
   this.sizer.margin = 6;
   this.sizer.spacing = 6;
   this.sizer.add ( this.helpLabel );
   this.sizer.spacing = 6;
   this.sizer.add ( this.fileSettings_GroupBox )
   this.sizer.spacing = 6;
   this.sizer.add ( this.Paths_GroupBox );
   this.sizer.addSpacing (4);
   this.sizer.add ( this.buttons_Sizer );

   this.windowTitle = TITLE + " Script v" + VERSION;
   this.adjustToContents();

}


wj_dialog.prototype = new Dialog;

function main()
{
   var dialog = new wj_dialog();
   dialog.execute();
}; // main()

main();

